package packed

import (
	"errors"
	"fmt"
	"github.com/gogf/gf/frame/g"
	"mime"
	"net/url"
	"path"
	"strings"
)

type file struct {
	url         string //课件地址
	ext         string //课件后缀
	name        string //课件名称
	nameHash    string //课件名hash
	contentHash string //课件内容hash
	dataPath    string //数据库存储路径
	fileInfoURL string //解析课件的URL
	tempZipPath string //临时存放zip包地址
	uploadPath  string //oss存储路径
	docType     string //课件type类型
	mineType    string //课件mine类型
}

func NewFile(ossFileLink, name string) (*file, error) {
	u, err := url.Parse(ossFileLink)
	if err != nil {
		g.Log().Async().Error(err)
		return nil, err
	}
	dataPath := u.EscapedPath()
	slicePath := strings.Split(strings.TrimLeft(dataPath, "/"), "/")

	if len(slicePath) < 4 {
		g.Log().Async().Error(errors.New("上传URL格式不正确"))
		return nil, errors.New("上传URL格式不正确")
	}
	f := &file{
		ext:         path.Ext(name),
		url:         ossFileLink,
		dataPath:    dataPath[1:],
		nameHash:    slicePath[3],
		contentHash: slicePath[2],
		name:        name,
		fileInfoURL: fmt.Sprintf("%s?info=0&ssl=1&furl=%s%s",
			ParserFileLink,
			g.Cfg().GetString("oss.url"),
			dataPath),
		tempZipPath: fmt.Sprintf("%s%s.zip",
			g.Cfg().GetString("temp.path"),
			slicePath[2]),
		uploadPath: strings.Join(slicePath[0:len(slicePath)-1], "/") + "/",
	}
	f.docType = f.getDocType()
	f.mineType = f.getMine()

	return f, nil
}

// 解析doctype
func (f *file) getDocType() string {
	switch f.ext {
	case ".ppt", "pptx":
		return "4"
	case ".doc", "docx":
		return "2"
	case "xls", "xlsx":
		return "3"
	case ".pdf":
		return "1"
	default:
		return "0"
	}
}

// 获取文件mine
func (f *file) getMine() string {
	return mime.TypeByExtension(f.ext)
}
