package packed

import (
	"encoding/json"
	"errors"
	"io/ioutil"
	"net/http"
	"net/http/cookiejar"
	"net/url"
	"regexp"
	"strings"

	"ppt_server/app/model"

	"github.com/gogf/gf/frame/g"
	"github.com/gogf/gf/util/gconv"
)

type Notfiy struct {
	name     string
	password string
	url      string
	c        *http.Client
}

func NewNotfiy() *Notfiy {
	jar, _ := cookiejar.New(nil)
	return &Notfiy{
		url:      g.Cfg().GetString("notify.url"),
		name:     g.Cfg().GetString("notify.name"),
		password: g.Cfg().GetString("notify.password"),
		c: &http.Client{
			Jar: jar,
			CheckRedirect: func(req *http.Request, via []*http.Request) error {
				return http.ErrUseLastResponse
			},
		},
	}
}

func (n *Notfiy) post(url string, data g.Map) error {

	res, err := n.request("POST", url, data)
	if err != nil {
		return err
	}
	var login model.NotifyResponse
	if err := json.Unmarshal(res, &login); err != nil {
		return err
	}

	if login.Code != 200 {
		return errors.New(login.Msg)
	}
	return nil
}

func (n *Notfiy) get(url string, data g.Map) ([]byte, error) {
	return n.request("GET", url, data)
}

func (n *Notfiy) request(method, r_url string, data g.Map) ([]byte, error) {

	vv := url.Values{}
	for k, v := range data {
		vv.Add(k, gconv.String(v))
	}
	vv.Encode()

	reader := strings.NewReader(vv.Encode())
	req, err := http.NewRequest(method, r_url, reader)
	if err != nil {
		return nil, err
	}
	req.Header.Set("Content-Type", "application/x-www-form-urlencoded")

	res, err := n.c.Do(req)
	if err != nil {
		return nil, err
	}
	defer res.Body.Close()

	return ioutil.ReadAll(res.Body)
}

func (n *Notfiy) auth() (string, error) {
	if err := n.post(n.url+"/web/login", g.Map{
		"name":     n.name,
		"password": n.password,
	}); err != nil {
		return "", err
	}

	// 获取Token
	tokenTypes, err := n.get(n.url+"/web/admin", nil)
	if err != nil {
		return "", err
	}
	re := regexp.MustCompile(`<meta name="csrf-token" content="([^"]+)">`)
	token := re.FindAllSubmatch(tokenTypes, -1)
	return string(token[0][1]), nil
}

// 上传失败通知
func (n *Notfiy) notify(path string, data g.Map) {

	csrfToken, err := n.auth()

	if err != nil {
		g.Log().Async().Error(err)
		return
	}
	data["_token"] = csrfToken
	if err = n.post(n.url+path, data); err != nil {
		g.Log().Async().Errorf("notify fail, err : %s\n", err)
	}
	g.Log().Async().Infof("Notify Success!")
	return
}

func (n *Notfiy) Fail(object *ObtainFile) {
	n.notify("/web/room_files_error", g.Map{
		"code":     400,
		"uuid":     object.r.Uuid,
		"room_num": object.r.RoomNum,
		"path":     object.parserFile.dataPath,
	})
}

func (n *Notfiy) Success(res *model.SuccessResponse) {

	n.notify("/web/room_files_add", gconv.Map(res))
}
